#!/bin/bash
# This assumes ssh public key exchange has been set up between the admin
# user (you) and both the root and BackupPC user on the BackupPC server.
# if not, it won't work.

#####################################################################
SERVER=moo.nac.uci.edu  # test with moo or real run with dabrick
#SERVER="dabrick.nacs.uci.edu"  # test with moo or real run with dabrick
# typically the BUPC_OWNER and the BACKUPPC_USER would be the same user
# usually backuppc. In my case for historical reasons, they're different.
# so you get the same option, but I'd rec that you stay with backuppc for both
BUPC_OWNER="hjm"
BACKUPPC_USER="backuppc"
WEB_GROUP="www-data"
#####################################################################

if [ ! $1 ]; then
    echo "Usage:"
    echo "  $0 backuppc_login_name   backuppc_password   FQ_home_dir   client_hostname  IP#  email"
    echo ""
    exit
fi

pause() {
    echo "======================================"
    echo $1
    echo "Hit 'Enter' to continue."
    echo "======================================"
    read input
}

# use to substitute the values in the .TMPL template files with those
# entered on the command line ie:
# ./converter.sh backuppc_login_name backuppc_password FQ_home_dir  client_hostname IP#  email
# ie: ./converter.sh hjm  pA55wrd0  /Users/hjm  booger.nacs.uci.edu 128.200.xxx.122 hjm@uci.edu
CL_USERNAME=$1
CL_PASSWORD=$2
CL_HOMEDIR=$3
CL_HOSTNAME=$4
CL_IPN=$5
CL_EMAIL=$6

# check to see if used '_' in hostname and convert to '-' to avoid problems with alive.py
# which breaks on '_'.

if [[ $CL_HOSTNAME =~ "_" ]]; then
	echo
	echo "INFO: Changing underscores to dashes in client:"
	echo -n "INFO: [$CL_HOSTNAME] ==> "
	# the '//' changes ALL instances
	CL_HOSTNAME=${CL_HOSTNAME//_/-}
	echo "[$CL_HOSTNAME]."
	echo "INFO: (Relax, will not affect other networking.)"
fi


echo
echo "This run is targetting server: [${SERVER}]"
echo
echo "USERNAME: $CL_USERNAME"
echo "PASSWORD: $CL_PASSWORD"
echo "HOMEDIR:  $CL_HOMEDIR"
echo "HOSTNAME: $CL_HOSTNAME"
echo "IP #:     $CL_IPN"
echo "EMAIL:    $CL_EMAIL"

# when testing strings, even the vers have to be quoted with "s
if [ "$CL_EMAIL" == "" -o "$7" != "" ]; then
  echo
  echo "ERROR: Wrong # of input parameters - please check the input vars carefully"
  echo
  exit 0
fi

pause "Above are the params I've got.  Make sure they're all filled out and aren't mixed up!"

echo
echo "Converting client files.."
echo
for OUTFILE in imalive.sh install_LinMac_backuppc_client.sh rsyncd.conf  rsyncd.secrets  rsync.password srvr_client_pc; do
   INFILE=${OUTFILE}.TMPL
   echo "converting [$INFILE] to [$OUTFILE]"  
   sed  "s/%%USERNAME%%/$CL_USERNAME/" < $INFILE | \
   sed  "s!%%HOMEDIR%%!$CL_HOMEDIR!"  | \
   sed  "s/%%PASSWORD%%/$CL_PASSWORD/" | \
   sed  "s/%%EMAIL%%/$CL_EMAIL/" | \
   sed  "s/%%HOSTNAME%%/$CL_HOSTNAME/"  > $OUTFILE
done

chmod 600 rsync.password  # chmod correctly or rsync won't go
chmod 600 rsyncd.secrets  # chmod correctly or rsync won't go

mv srvr_client_pc server/${CL_HOSTNAME}.pl # now that it's substituted, can just copy direct to server files


echo
echo "Converting server files.."
echo
# need to process: /etc/hosts /etc/BackupPC/hosts /etc/BackupPC/rsyncd.secrets htpasswd.4.backuppc

cd server 

echo "Server is: [${SERVER}]"
echo
echo "Pushing  srvr_client_pc to ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/pc/${CL_HOSTNAME}.pl"
scp ${CL_HOSTNAME}.pl ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/pc/
sudo ssh ${SERVER} "chown ${BACKUPPC_USER}.${WEB_GROUP} /etc/BackupPC/pc/${CL_HOSTNAME}.pl; chmod 640  /etc/BackupPC/pc/${CL_HOSTNAME}.pl"

echo ""
echo "Processing ${SERVER}:/etc/hosts"
# process /etc/hosts -> etc.hosts
scp ${BUPC_OWNER}@${SERVER}:/etc/hosts etc.hosts # get latest
# next line splits HOSTNAME into myArr on '.' 
grep -v '#### BackupPC hosts END' etc.hosts > tmp # kill the last line
read -a myArr <<<${CL_HOSTNAME//./ } 
SHORTHOST=${myArr[0]}                                                                                                        
echo "$CL_IPN   $CL_HOSTNAME   $SHORTHOST" >>tmp
echo "#### BackupPC hosts END - DO NOT MODIFY THIS LINE ####" >> tmp
mv tmp etc.hosts
echo "pushing new /etc/hosts back as '/etc/hosts.cand'"
sudo scp etc.hosts ${SERVER}:/etc/hosts.cand # copy into place next to operating one
echo "Here is the server-side diff between /etc/hosts & /etc/hosts.cand"
echo "=============================================================="
sudo ssh ${SERVER} 'diff /etc/hosts /etc/hosts.cand'
echo "=============================================================="
pause "Check diffs.  Hit [Enter] to replace, [Cntrl+C] to cancel"
# set permissions to root.root: -rw-r--r--
sudo ssh ${SERVER} 'mv /etc/hosts /etc/hosts.bak; mv /etc/hosts.cand /etc/hosts; chmod 644 /etc/hosts'
echo "'/etc/hosts' Done"
echo

# process /etc/BackupPC/hosts
echo "Processing ${SERVER}:/etc/BackupPC/hosts"
scp ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/hosts hosts # get latest
# append the line NB: '1' indicates DHCP, generally the case.
# but in the case where we're using the imalive system, they should be
# considered static IPs (in /etc/hosts BackupPC stanza)
echo "$CL_HOSTNAME   0   $CL_USERNAME #from LinMac_Client_Config.sh" >> hosts
echo "Pushing 'hosts' to ${SERVER}:/etc/BackupPC/"
scp hosts ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/hosts  # push it back 
# set ownership and perms. hjm www-data: -rw-r--r-- 1 
sudo ssh ${SERVER} 'chown ${BUPC_OWNER}.${WEB_GROUP} /etc/BackupPC/hosts; chmod 644 /etc/BackupPC/hosts'
echo "'/etc/BackupPC/hosts' Done" 
echo

# process /etc/BackupPC/rsyncd.secrets
echo "Processing ${SERVER}:/etc/BackupPC/rsyncd.secrets"
scp ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/rsyncd.secrets . # get latest
# append the line
echo "${CL_USERNAME}:${CL_PASSWORD}" >>  rsyncd.secrets
echo "Pushing 'rsyncd.secrets' to ${SERVER}:/etc/BackupPC"
scp rsyncd.secrets ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/rsyncd.secrets  # push it back 
sudo ssh ${SERVER} 'chown ${BUPC_OWNER}.${WEB_GROUP} /etc/BackupPC/rsyncd.secrets; chmod 600 /etc/BackupPC/rsyncd.secrets'
echo "'/etc/BackupPC/rsyncd.secrets' Done" 
echo

# process htpasswd.4.backuppc
echo "Processing ${SERVER}:/etc/BackupPC/htpasswd.4.backuppc"
scp ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/htpasswd.4.backuppc . # get latest
htpasswd -b ./htpasswd.4.backuppc $CL_USERNAME $CL_PASSWORD
echo "Pushing 'htpasswd.4.backuppc' to ${SERVER}:/etc/BackupPC"
scp htpasswd.4.backuppc ${BUPC_OWNER}@${SERVER}:/etc/BackupPC/htpasswd.4.backuppc  # push it back 
sudo ssh ${SERVER} 'chown ${BUPC_OWNER} .${WEB_GROUP} /etc/BackupPC/htpasswd.4.backuppc; chmod 660 /etc/BackupPC/htpasswd.4.backuppc'
echo "'/etc/BackupPC/rsyncd.secrets' Done" 
echo ""

cd ../..

if [ -e ${CL_USERNAME}_${CL_HOSTNAME}.zip ]; then 
  echo "existing zip file [${CL_USERNAME}_${CL_HOSTNAME}.zip]; deleting it."
  rm ${CL_USERNAME}_${CL_HOSTNAME}.zip # make sure we make a new one rather than updating an old one
fi
echo "Now packaging client files into ${CL_USERNAME}_${CL_HOSTNAME}.zip"

# # use explict filenames, so as to prevent accidentally leaking info.
zip -vr ${CL_USERNAME}_${CL_HOSTNAME}.zip \
   backuppc/rsyncd.exclude \
   backuppc/SLA.txt \
   backuppc/rsync.password \
   backuppc/srvr_client_pc \
   backuppc/rsync.plist \
   backuppc/README \
   backuppc/rsync_fat.gz \
   backuppc/install_LinMac_backuppc_client.sh \
   backuppc/imalive.sh \
   backuppc/rsyncd.secrets \
   backuppc/Mac_Linux_Email_Instructions.html \
   backuppc/Mac_Linux_Email_Instructions.txt \
   backuppc/rsync.init \
   backuppc/rsyncd.conf

echo""
PWD=`pwd`
echo "Attach the following files to an email to ${CL_EMAIL}"
echo "${PWD}/${CL_USERNAME}_${CL_HOSTNAME}.zip"
echo "${PWD}/backuppc/Mac_Linux_Email_Instructions.txt"
echo "${PWD}/backuppc/Mac_Linux_Email_Instructions.html"
# if you have a working mail system, you can automate this via mutt 
# (uncomment the lines below) or another mailer.
# mutt -a ${PWD}/${CL_USERNAME}_${CL_HOSTNAME}.zip \
# -a ${PWD}/Mac_Linux_Email_Instructions.txt \
# -a ${PWD}/Mac_Linux_Email_Instructions.html \
# -s "Your BackupPC installation script & Instructions" ${CL_EMAIL}

echo ""
echo "NOW we're all done!!"
