#!/bin/sh
# 2 lines below must be substituted
BUPC_USER=hjmangalam
BUPC_HOST=harrys-big-banger.nukes.uci.edu

# this is simple enough that it can be run for both Mac & Linux
# this is the OSX version only bc of the RSYNC_BIN def 
# since OSX has the broken 2.6.9 version
RSYNC_BIN="/usr/bin/rsync"
INSTALL_DIR='/etc/BackupPC' # this is the same for Mac (L&T) and Linux

# BUPC_HOST has to come from the form bc the host-supplied one via `hostname`
# will often be useless (dchp-32.dept.uci.edu instead of princess.dept.uci.edu)
# so the hostname is merely an identifier rather than a FQDN-type host.
# and trying to use it for anything else (such as to ping or ssh to it) will fail
IPFILENAME=$BUPC_HOST'_'$BUPC_USER'_dhcp'   # for name like 'flop.nac.uci.edu_hjmangalam_dhcp'

# following function verifies valid IP#s and only to UCI networks
function valid_ip() {
    local  ip=$1
    local  stat=1
    
    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        # 1st 3 lines restricts to UCI IP #s
		#128.195.0.0/16 campus
		#128.200.0.0/16 campus
		#169.234.0.0/16 ResNet, Mobile Access (WiFi), VPN
		[[ ${ip[0]} -eq 128 && ${ip[1]} -eq 200 \
		|| ${ip[0]} -eq 128 && ${ip[1]} -eq 195 \
		|| ${ip[0]} -eq 169 && ${ip[1]} -eq 234 \
        && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}


UNAME=`uname`
if [ $UNAME = 'Linux' ]; then
	for dev in eth0 eth1 wlan0 wlan1 ath0 ath1; do
		MYIP=`ifconfig $dev 2> /dev/null |grep 'inet addr' |cut -f2 -d':'| cut -f1 -d' '`
# next line is for debugging the regex
#		if [ $dev = 'eth1' ]; then MYIP='129.195.34.211'; fi
		if valid_ip $MYIP ; then
			echo "Found a valid UCI net address on $dev : $MYIP"
			echo $MYIP >$INSTALL_DIR/$IPFILENAME
			break
		fi
	done
fi

if [ $UNAME = 'Darwin' ]; then
	# search thru the possible MacOSX dev names
	for dev in en0 en1 en2 en3; do
		MYIP=`/sbin/ifconfig $dev 2> /dev/null|grep 'inet ' |cut -f2 -d' '`
# next line is for debugging the regex
#		if [ $dev = 'eth1' ]; then MYIP='129.195.34.211'; fi
		if valid_ip $MYIP ; then
			echo "Found a valid UCI net address on $dev : $MYIP"
			echo $MYIP >$INSTALL_DIR/$IPFILENAME
			break
		fi
	done
fi

IPFILENAME=$BUPC_HOST'_'$BUPC_USER'_dhcp'   # for name like 'flop.nac.uci.edu_hjmangalam_dhcp'

# following assumes that it will be run out of $INSTALL_DIR and puts the
# file into the "alive" share, which will be /etc/BackupPC/alive dir on server.
$RSYNC_BIN --password-file $INSTALL_DIR/rsync.password $INSTALL_DIR/$IPFILENAME ${BUPC_USER}@128.200.84.82::alive
