#!/usr/bin/python
"""
Part of the WebBUPC SERVER package; Script to go thru all the DHCP files in
the /etc/BackupPC/alive dir to update the info in the server's /etc/hosts
file to correct for DHCP-supplied IP numbers.  This allows the BackupPC
server to get to clients who have had recent IP changes.  Even over the
Internet, if we want to allow such a service.
This script should be run from cron ~ every 15 min with the following line:
3,18,33,48 *    * * *   root    cd /etc/BackupPC/alive && ./alive.py
25,26,27,28,29,30 *    * * *   root    cd /home/hjm/nacs/webbackuppc/alive && ./alive.py

"""
# changes:
# 7.21.11 - changed the split to rsplit to handle bad hostnames (which have '_' intead of '-'
#   ie LisaFoo_Personal_Desktop.
#   this bug-check is coded into the LinMac config shell script so it shouldn't matter going forward
#   but JIC..  It also retroactively allows alive.py to process 'bad' hostnames.

import glob
import re
import sys,os
from datetime import datetime

# these vars are meant to be relative to the installation dir.
HOSTSFILE = "/etc/hosts"  #"./dabrickhosts"
ALIVE_DIR = "/etc/BackupPC/alive"
TMPHOSTS = "/tmp/hosts"
BACKUP_DIR = "/etc/BackupPC/backup"
session_date = datetime.now().strftime("%Y%m%dT%H%M%S") # timestamp for session
DEBUG=0 # set to 1 for diagnostic print statements

# /etc/hosts contents (after delimiter) should be read in as a hash,
# indexed by the IPnbr for later  use ^####BackupPC BEGIN#### as delim
host_dict = {}
# /etc/hosts file HAS to have these lines inserted prior to regular running of the script
BUPC_BEGIN = "#### BackupPC hosts BEGIN - DO NOT MODIFY THIS LINE ####"
BUPC_END   = "#### BackupPC hosts END - DO NOT MODIFY THIS LINE ####"
inbupc = 0
newhosts = ""
etchosts = file(HOSTSFILE, "r") # open the file

# Load all the previous BUPC client info into a dict for later comparison to
# that extracted from the BUPC clients.  NON-BUPC data loaded into 'newhosts' string
# BUPC-related data loaded into host_dict
for line in etchosts:
    if line and re.search(BUPC_BEGIN, line): # skip down to delimiter
        newhosts += line # keep the stanza header
        inbupc = 1
        continue # not break - 'break' breaks out of loop
    if line and re.search(BUPC_END, line):
        newhosts += line # keep the stanza header
        inbupc = 0
        continue
    if line and inbupc == 0:
        newhosts += line
    if line and inbupc: # now process all the lines until inbupc is zeroed
        # all the lines in here should be machine-generated and consist of IPnbr___hostname___host
        # where ___ is whitespace.
        IPnbr, hostname, host = line.split()
        if DEBUG: print IPnbr, "\t", hostname, "\t",  host
        host_dict[hostname] = (IPnbr, host) # recall with host_dict[hostname][0] -> IPnbr
        if DEBUG: print "IPN=", host_dict[hostname][0], "host=", host_dict[hostname][1]
        # NB: can also make 2-key dicts with: host_dict[(IPnbr,hostname)] = host
etchosts.close()
if DEBUG: print "Done with processsing /etc/hosts"
#response = raw_input ("Waiting")

# now process all the 'alive' DHCP files returned from the BUPC clients,
# loading new/different data into same host_dict as above.
fqp2dhcp_lst = glob.glob(ALIVE_DIR + '/*_dhcp') # only need to proc dhcp files.
for flnme in fqp2dhcp_lst:
    if DEBUG: print "\nfor: ", flnme
    # now get contents of the file - will be either the IPnbr or a phrase including the IPnbr.
    fp = file(flnme, "r") # open the file
    for line in fp:
        line =  line.strip()
        if re.search("Address",line): # from a Windows (2K, XP, Vista OK) client; has to be slightly processed
            IPnbr = line.split(':')[-1].strip()
        else: # it's a Linux or Mac client (simple, isolated IP #)
            IPnbr = line.strip()
    if DEBUG: print "IP #: ", IPnbr
    fn = flnme.split('/')[-1] # split path name & naked filename is last el
    hostname, user, dhcp = fn.rsplit('_',2) # and we don't need the last el (dhcp) cuz that's how we selected it.
    host, subnet, net, domain = hostname.split('.')
    # should now read the real /etc/hosts file  and insert the appro line instead.
    try:
        host_dict[hostname][0]
        if DEBUG: print "Defined"
        if host_dict[hostname][0] == IPnbr:
            if DEBUG: print "IP #s match"
            if DEBUG: sys.stderr.write("INFO: MATCH-[%s] matches hosts file already;\n\tNOT changed\n" % (flnme))
        else:
            if DEBUG: print "IP#s DON'T MATCH, so have to change as if they weren't defined."
            host_dict[hostname]= (IPnbr, host)
    except: # get here if it's undefined
        if DEBUG: sys.stderr.write("INFO: NO MATCH-[%s] DOESN'T match hosts file;\n\tNEW DATA INSERTED in dict\n" % (flnme))
        host_dict[hostname]= (IPnbr, host) # so insert it into the dict
if DEBUG: print "Done loading host_dict"
#response = raw_input ("Waiting")

hostlines = newhosts.splitlines(True) # transfer newhosts data into line-by-line data
# and finally rewrite the hosts file dumping the host_dict into the bupc stanza
# NON-BUPC data from 'hostlines' imm above), BUPC data from 'host_dict'
#
inbupc = 0
newhosts = "" # re-use newhosts as container for updated info
for line in hostlines:
    if line and re.search(BUPC_BEGIN, line):
        newhosts += line
        inbupc = 1
        for hostname in host_dict.iterkeys(): # then print out the dict
            IPnbr = host_dict[hostname][0]
            host =  host_dict[hostname][1]
            newhosts += "%s    \t%s    \t%s\n" % (IPnbr, hostname, host)
        continue
    if line and re.search(BUPC_END, line):
        newhosts += line
        inbupc = 0
        continue
    if line and inbupc == 0:
        newhosts += line

#print "new hosts file: ", newhosts
tmphosts = file(TMPHOSTS,"w")
tmphosts.write(newhosts) # write it out.
tmphosts.close()

# now move original hosts file out of the way and write in a new one
# with the contents of newhosts.
# cp the original HOSTSFILE to BACKUP_DIR (/etc/BackupPC/backup) as etchosts.<timestamp>
backup_hosts_flnme = BACKUP_DIR + '/' + "etchosts." + session_date
if DEBUG: print "backup_hosts_flnme=", backup_hosts_flnme
#cpcmd = "cp %s %s" % (HOSTSFILE, backup_hosts_flnme)
#os.system(cpcmd)
# mv the TMPHOSTS file to HOSTSFILE
mvcmd = "mv  %s  %s" % (TMPHOSTS, HOSTSFILE)
os.system(mvcmd)
