import turbogears
from formencode.api import Invalid
import formencode
import re
from sqlobject import *
import model
#for debugging purposes, can be removed after reaching a stable state.
from IPython.Shell import IPShellEmbed
# and insert 'dbg()' where you want to break.
dbg = IPShellEmbed() 

class SecurePassword(turbogears.validators.TgFancyValidator):
	min = 3
	non_letter = 1
	letter_regex = re.compile(r'[a-zA-Z]')
	messages = {'too_few': 'Your password must be longer than %(min)i ' 'characters long', 'non_letter': 'You must include at least %(non_letter)i ' 'characters in your password', }
	def _to_python(self, value, state):
		return value.strip()
	def validate_python(self, value, state):
		if len(value) < self.min:
			raise Invalid(self.message("too_few", state, min=self.min), value, state)
		non_letters = self.letter_regex.sub('', value)
		if len(non_letters) < self.non_letter:
			raise Invalid(self.message("non_letter",non_letter=self.non_letter),value, state)
class TrueValidator(turbogears.validators.TgFancyValidator):
	messages = {'agree': 'You did not agree to the SLA - please read it and resubmit!'}
	def _to_python(self, value, state):
		return value
	def validate_python(self, value, state):
		if value == False:
			raise Invalid(self.message("agree",state), value, state)
class IpValidator(turbogears.validators.TgFancyValidator):
	messages = {'alreadyExists': 'There is an entry with that IP!',
		    'invalidFormat':'The IP number must be four numbers separated with "."',
	            'firstNumberError':'first number must be either 128 or 129',
		    'secondNumberError':'Second number must be either 200,195 or 148',
		    'thirdNumberError':'Third number must be between 0 and 255',
		    'fourthNumberError':'Fourth number must be between 1 and 254'
	    }
	num1=0
	num2=0
	num3=0
	num4=0
	def _to_python(self, value, state):
		return value
	def validate_python(self, value, state):
		tokens = value.split('.')
		if len(tokens)!=4:
			raise Invalid(self.message("invalidFormat",state), value, state)
		else:
			if not tokens[0] == '128' and not tokens[0] == '129':
				raise Invalid(self.message("firstNumberError",state), value, state)
			if not tokens[1] == '200' and not tokens[1] == '195' and not tokens[1]=='148':
				raise Invalid(self.message("secondNumberError",state), value, state)
			if int(tokens[2]) > 255 or int(tokens[2]) < 0:
				raise Invalid(self.message("thirdNumberError",state), value, state)				
			if int(tokens[3]) > 254 or int(tokens[3]) < 1:
				raise Invalid(self.message("fourthNumberError",state), value, state)				
		try :
			pcInfo = model.pcinfo.byIP(value)
			raise Invalid(self.message("alreadyExists",state), value, state)
		except SQLObjectNotFound:		
			pcInfo = "dummy"
			
class HostValidator(turbogears.validators.TgFancyValidator):
	messages = {'alreadyExists': 'There is an entry with that Hostname!','wrongDomain':"The HOSTNAME must end with 'uci.edu'."}
	def _to_python(self, value, state):
		return value
	def validate_python(self, value, state):
		try :
			pcInfo = model.pcinfo.byHost(value)
			raise Invalid(self.message("alreadyExists",state), value, state)
		except SQLObjectNotFound:		
			pcInfo = "dummy"
		if not value.endswith('uci.edu'):
			raise Invalid(self.message("wrongDomain",state), value, state)

class UsernameValidator(turbogears.validators.TgFancyValidator):
	min = 3
	messages = {'too_few': 'Your username must be longer than %(min)i ' 'characters long', 'alreadyExists': 'There is an entry with that Username!'}
	def _to_python(self, value, state):
		return value	
	def validate_python(self, value, state):
		if len(value) < self.min:
			raise Invalid(self.message("too_few", state, min=self.min), value, state)

class AlreadyExistValidator(formencode.validators.FormValidator):
	show_match = False
	field_names = None
	validate_partial_form = True
	__unpackargs__ = ('*', 'field_names')
	messages = {
        'differentPasswordInFile': "We have a different password in file for that username.",
        'UsernameTaken': "Username is taken",
        }
	
	def validate_partial(self, field_dict, state):
		for name in self.field_names:
			if not field_dict.has_key(name):
				return
			self.validate_python(field_dict, state)
	def validate_python(self, field_dict, state):
		user = field_dict['USERNAME']
		password = field_dict['PASSWORD1']
		firstName = field_dict['FIRSTNAME']
		lastName = field_dict['LASTNAME']
		try:
			userInfo = model.owner.byUsername(user)
			errors = {}
			if user == userInfo.UserID:
				if firstName == userInfo.FirstName:
					if lastName==userInfo.LastName:
						if password == userInfo.Password:
							dummy="all information matches, no problem"
						else:
							errors['PASSWORD1'] = self.message('differentPasswordInFile',state)
					else:
						errors['USERNAME'] = self.message('UsernameTaken',state)
				else:
					errors['USERNAME'] = self.message('UsernameTaken',state)
			
			if errors:
				error_list = errors.items()
				error_list.sort()
				error_message = '<br>\n'.join(
					['%s: %s' % (name, value) for name, value in error_list])
				raise Invalid(error_message,
					      field_dict, state,
					      error_dict=errors)
		except SQLObjectNotFound:
			dummy="No owners with that username!!! ITS VALID!"
		
#Class to handle the password validation.
#**
class BUPCFormSchema(turbogears.validators.Schema):
    PASSWORD2 = turbogears.validators.UnicodeString(min=6)
    PASSWORD2 = turbogears.validators.UnicodeString(min=6)
    USERNAME=UsernameValidator()
    chained_validators = [turbogears.validators.FieldsMatch('PASSWORD1', 'PASSWORD2'), AlreadyExistValidator('PASSWORD1','USERNAME','FIRSTNAME','LASTNAME')]
    
			
		
